=== added file 'bin/ivle-refreshfilesystem'
--- bin/ivle-refreshfilesystem	1970-01-01 00:00:00 +0000
+++ bin/ivle-refreshfilesystem	2009-05-11 04:51:08 +0000
@@ -0,0 +1,38 @@
+#!/usr/bin/env python
+# IVLE - Informatics Virtual Learning Environment
+# Copyright (C) 2007-2009 The University of Melbourne
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
+"""Refresh parts of the filesystem that are generated from the database.
+
+In particular, the Subversion authorisation files are rewritten.
+"""
+
+import sys
+
+from ivle.config import Config
+from ivle.database import get_store
+import ivle.makeuser
+
+config = Config()
+store = get_store(config)
+
+print >>sys.stderr, "Rebuilding Subversion user configuration..."
+ivle.makeuser.rebuild_svn_config(store, config)
+print >>sys.stderr, "Rebuilding Subversion group configuration..."
+ivle.makeuser.rebuild_svn_group_config(store, config)
+
+

=== modified file 'ivle/makeuser.py'
--- ivle/makeuser.py	2009-04-30 01:51:44 +0000
+++ ivle/makeuser.py	2009-05-11 04:28:07 +0000
@@ -27,8 +27,11 @@
 import logging
 import subprocess
 
+from storm.expr import Select, Max
+
 import ivle.config
-from ivle.database import ProjectGroup, User
+from ivle.database import (User, ProjectGroup, Assessed, ProjectSubmission,
+        Project, ProjectSet, Offering, Enrolment, Subject, Semester)
 
 def chown_to_webserver(filename):
     """chown a directory and its contents to the web server.
@@ -71,6 +74,49 @@
 %(login)s = rw
 """ % {'login': u.login})
 
+    # Now we need to grant offering tutors and lecturers access to the latest
+    # submissions in their offerings. There are much prettier ways to do this,
+    # but a lot of browser requests call this function, so it needs to be
+    # fast. We can grab all of the paths needing authorisation directives with
+    # a single query, and we cache the list of viewers for each offering.
+    offering_viewers_cache = {}
+    for (login, psid, pspath, offeringid) in store.find(
+        (User.login, ProjectSubmission.id, ProjectSubmission.path,
+         Offering.id),
+            Assessed.id == ProjectSubmission.assessed_id,
+            User.id == Assessed.user_id,
+            Project.id == Assessed.project_id,
+            ProjectSet.id == Project.project_set_id,
+            Offering.id == ProjectSet.id,
+            ProjectSubmission.date_submitted == Select(
+                    Max(ProjectSubmission.date_submitted),
+                    ProjectSubmission.assessed_id == Assessed.id,
+                    tables=ProjectSubmission
+            )
+        ):
+
+        # Do we already have the list of logins authorised for this offering
+        # cached? If not, get it.
+        if offeringid not in offering_viewers_cache:
+            offering_viewers_cache[offeringid] = list(store.find(
+                    User.login,
+                    User.id == Enrolment.user_id,
+                    Enrolment.offering_id == offeringid,
+                    Enrolment.role.is_in((u'tutor', u'lecturer'))
+                )
+            )
+
+        f.write("""
+# Submission %(id)d
+[%(login)s:%(path)s]
+""" % {'login': login, 'id': psid, 'path': pspath})
+
+        for viewer_login in offering_viewers_cache[offeringid]:
+            # We don't want to override the owner's write privilege,
+            # so we don't add them to the read-only ACL.
+            if login != viewer_login:
+                f.write("%s = r\n" % viewer_login)
+
     f.close()
     os.rename(temp_name, conf_name)
     chown_to_webserver(conf_name)
@@ -90,6 +136,7 @@
 
 """ % {'time': time.asctime()})
 
+    group_members_cache = {}
     for group in store.find(ProjectGroup):
         offering = group.project_set.offering
         reponame = "_".join([offering.subject.short_name,
@@ -98,10 +145,60 @@
                              group.name])
 
         f.write("[%s:/]\n" % reponame)
+        if group.id not in group_members_cache:
+            group_members_cache[group.id] = set()
         for user in group.members:
+            group_members_cache[group.id].add(user.login)
             f.write("%s = rw\n" % user.login)
         f.write("\n")
 
+    # Now we need to grant offering tutors and lecturers access to the latest
+    # submissions in their offerings. There are much prettier ways to do this,
+    # but a lot of browser requests call this function, so it needs to be
+    # fast. We can grab all of the paths needing authorisation directives with
+    # a single query, and we cache the list of viewers for each offering.
+    offering_viewers_cache = {}
+    for (ssn, year, sem, name, psid, pspath, gid, offeringid) in store.find(
+        (Subject.short_name, Semester.year, Semester.semester,
+         ProjectGroup.name, ProjectSubmission.id, ProjectSubmission.path,
+         ProjectGroup.id, Offering.id),
+            Assessed.id == ProjectSubmission.assessed_id,
+            ProjectGroup.id == Assessed.project_group_id,
+            Project.id == Assessed.project_id,
+            ProjectSet.id == Project.project_set_id,
+            Offering.id == ProjectSet.offering_id,
+            Subject.id == Offering.subject_id,
+            Semester.id == Offering.semester_id,
+            ProjectSubmission.date_submitted == Select(
+                    Max(ProjectSubmission.date_submitted),
+                    ProjectSubmission.assessed_id == Assessed.id,
+                    tables=ProjectSubmission
+            )
+        ):
+
+        reponame = "_".join([ssn, year, sem, name])
+
+        # Do we already have the list of logins authorised for this offering
+        # cached? If not, get it.
+        if offeringid not in offering_viewers_cache:
+            offering_viewers_cache[offeringid] = list(store.find(
+                    User.login,
+                    User.id == Enrolment.user_id,
+                    Enrolment.offering_id == offeringid,
+                    Enrolment.role.is_in((u'tutor', u'lecturer'))
+                )
+            )
+
+        f.write("""
+# Submission %(id)d
+[%(repo)s:%(path)s]
+""" % {'repo': reponame, 'id': psid, 'path': pspath})
+
+        for viewer_login in offering_viewers_cache[offeringid]:
+            # Skip existing group members, or they can't write to it any more.
+            if viewer_login not in group_members_cache[gid]:
+                f.write("%s = r\n" % viewer_login)
+
     f.close()
     os.rename(temp_name, conf_name)
     chown_to_webserver(conf_name)

=== modified file 'ivle/webapp/submit/__init__.py'
--- ivle/webapp/submit/__init__.py	2009-05-05 06:37:51 +0000
+++ ivle/webapp/submit/__init__.py	2009-05-12 08:29:47 +0000
@@ -31,6 +31,7 @@
 from ivle.webapp.base.plugins import ViewPlugin
 
 import ivle.date
+import ivle.chat
 
 
 class SubmitView(XHTMLView):
@@ -92,6 +93,33 @@
 
             project.submit(self.context, self.path, revision, req.user)
 
+            # The Subversion configuration needs to be updated, to grant
+            # tutors and lecturers access to this submission. We have to 
+            # commit early so usrmgt-server can see the new submission.
+            req.store.commit()
+
+            # Instruct usrmgt-server to rebuild the SVN group authz file.
+            msg = {'rebuild_svn_group_config': {}}
+            usrmgt = ivle.chat.chat(req.config['usrmgt']['host'],
+                                    req.config['usrmgt']['port'],
+                                    msg,
+                                    req.config['usrmgt']['magic'],
+                                    )
+
+            if usrmgt.get('response') in (None, 'failure'):
+                raise Exception("Failure creating repository: " + str(usrmgt))
+
+            # Instruct usrmgt-server to rebuild the SVN user authz file.
+            msg = {'rebuild_svn_config': {}}
+            usrmgt = ivle.chat.chat(req.config['usrmgt']['host'],
+                                    req.config['usrmgt']['port'],
+                                    msg,
+                                    req.config['usrmgt']['magic'],
+                                    )
+
+            if usrmgt.get('response') in (None, 'failure'):
+                raise Exception("Failure creating repository: " + str(usrmgt))
+
             self.template = 'submitted.html'
             ctx['project'] = project
 

=== modified file 'setup/util.py'
--- setup/util.py	2009-05-11 05:27:56 +0000
+++ setup/util.py	2009-05-12 08:44:07 +0000
@@ -324,4 +324,5 @@
         "bin/ivle-buildjail",
         "bin/ivle-addexercise",
         "bin/ivle-cloneworksheets",
+        "bin/ivle-refreshfilesystem",
     ]

